import { ɵɵdefineInjectable, ɵɵinject, Injectable, Inject, InjectionToken } from '@angular/core';
import { Router } from '@angular/router';
import { Subject, merge } from 'rxjs';
import { map, auditTime, takeUntil } from 'rxjs/operators';
import set from 'lodash.set';

function parse(value, type) {
    switch (type) {
        case 'string':
            return value;
        case 'boolean':
            return value === 'false' ? false : !!value;
        case 'array':
            return value === null || value === void 0 ? void 0 : value.split(',');
        case 'number':
            return +value;
        default:
            return value;
    }
}
function get(obj, path) {
    let current = obj;
    path.split('.').forEach((p) => (current = current[p]));
    return current;
}
function resolveParams(params) {
    const toArray = coerceArray(params);
    const result = {};
    toArray.forEach(({ value, queryKey }) => {
        const isEmpty = value === null || value === undefined || !value.toString();
        result[queryKey] = isEmpty ? null : value.toString();
    });
    return result;
}
function coerceArray(value) {
    return Array.isArray(value) ? value : [value];
}
function defsToParams(defs, group) {
    return defs.map((def) => {
        return {
            queryKey: def.queryKey,
            value: group.get(def.path).value,
        };
    });
}

class QueryParamDef {
    constructor(config) {
        this.config = config;
    }
    get queryKey() {
        return this.config.queryKey;
    }
    get path() {
        return this.config.path || this.queryKey;
    }
    get type() {
        return this.config.type || 'string';
    }
    get strategy() {
        return this.config.strategy || 'twoWay';
    }
    parse(queryParamValue) {
        return parse(queryParamValue, this.type);
    }
}
class BindQueryParamsFactory {
    constructor(router, options) {
        this.router = router;
        this.options = options;
    }
    create(defs, group) {
        return new BindQueryParamsManager(this.router, this.options).connect(defs, group);
    }
}
BindQueryParamsFactory.ɵprov = ɵɵdefineInjectable({ factory: function BindQueryParamsFactory_Factory() { return new BindQueryParamsFactory(ɵɵinject(Router), ɵɵinject(BIND_QUERY_PARAMS_OPTIONS)); }, token: BindQueryParamsFactory, providedIn: "root" });
BindQueryParamsFactory.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
BindQueryParamsFactory.ctorParameters = () => [
    { type: Router },
    { type: undefined, decorators: [{ type: Inject, args: [BIND_QUERY_PARAMS_OPTIONS,] }] }
];
class BindQueryParamsManager {
    constructor(router, options) {
        this.router = router;
        this.options = options;
        this.$destroy = new Subject();
    }
    connect(defs, group) {
        this.defs = coerceArray(defs).map((def) => new QueryParamDef(def));
        this.group = group;
        this.onInit();
        return this;
    }
    onInit() {
        const value = this.getInitialValue();
        if (Object.keys(value).length) {
            this.group.patchValue(value);
        }
        const controls = this.defs.map((def) => {
            return this.group.get(def.path).valueChanges.pipe(map((value) => ({
                value,
                queryKey: def.queryKey,
            })));
        });
        // Could be a several changes in the same tick,
        // for example when we use reset() or patchValue.
        // We need to aggregate the changes and apply them at once
        // because the router navigates in micro task
        let buffer = [];
        merge(...controls)
            .pipe(map((result) => buffer.push(result)), auditTime(0), takeUntil(this.$destroy))
            .subscribe(() => {
            this.updateQueryParams(resolveParams(buffer));
            buffer = [];
        });
    }
    destroy() {
        this.$destroy.next();
    }
    getDef(queryKey) {
        return this.defs.find((def) => def.queryKey === queryKey);
    }
    parse(queryParams) {
        const result = {};
        for (const [key, value] of Object.entries(queryParams)) {
            const def = this.getDef(key);
            if (def) {
                result[key] = this.getDef(key).parse(value);
            }
        }
        return result;
    }
    updateQueryParams(queryParams) {
        this.router.navigate([], {
            queryParams,
            queryParamsHandling: 'merge',
            replaceUrl: true,
        });
    }
    getInitialValue() {
        const queryParams = new URLSearchParams(this.options.windowRef.location.search);
        let value = {};
        for (const def of this.defs) {
            if (def.strategy === 'twoWay') {
                const queryKey = def.queryKey;
                if (queryParams.has(queryKey)) {
                    set(value, def.path, def.parse(queryParams.get(queryKey)));
                }
            }
        }
        return value;
    }
}
const BIND_QUERY_PARAMS_OPTIONS = new InjectionToken('BIND_QUERY_PARAMS_OPTIONS', {
    providedIn: 'root',
    factory() {
        return {
            windowRef: window,
        };
    },
});

/**
 * Generated bundle index. Do not edit.
 */

export { BIND_QUERY_PARAMS_OPTIONS, BindQueryParamsFactory };
//# sourceMappingURL=ngneat-bind-query-params.js.map
